/*
  Pidgin-Audacious-Remote - Control audacious from Pidgin conversations
  Copyright (C) 2004 Gary Kramlich
  Copyright (C) 2007 Francesco Russo

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <string.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include <audacious/audctrl.h>
#include <audacious/dbus.h>
/* #include <audacious/titlestring.h> */

#define PURPLE_PLUGINS
#include <cmds.h>
#include <conversation.h>
#include <gtkconv.h>
#include <gtkmenutray.h>
#include <gtkplugin.h>
#include <gtkprefs.h>
#include <gtkutils.h>
#include <gtkblist.h>
#include <version.h>
#include <debug.h>

/****************************************************************************
 * Constants
 ***************************************************************************/
#define PAR_VERSION VERSION
/* #define PAR_SESSION (purple_prefs_get_int("/plugins/gtk/pidgin-audacious-remote/session")) */
#define PAR_STOCK_NEXT "par-next"
#define PAR_STOCK_PAUSE "par-pause"
#define PAR_STOCK_PLAY "par-play"
#define PAR_STOCK_PREVIOUS "par-previous"
#define PAR_STOCK_STOP "par-stop"
#define PAR_STOCK_AUDACIOUS "par-audacious"
#define PAR_DBUS_TIMEOUT (5*1000)
#define par_debug(fmt, ...) purple_debug(PURPLE_DEBUG_INFO, "Pidgin-Audacious", fmt, ## __VA_ARGS__);

enum
{PAR_PREF_PLAYLIST = 1,
 PAR_PREF_BLIST,
 PAR_PREF_CONV_WINDOW,
 PAR_PREF_EXTENDED_CTRL,
 PAR_PREF_VOLUME_CTRL
};

/****************************************************************************
 * Globals
 ***************************************************************************/
static GList *buttons = NULL, *checkboxes = NULL;
static GtkIconFactory *icon_factory;
static GtkWidget *blist_button = NULL;
static PurpleCmdId par_cmd;
static DBusGProxy *PAR_SESSION;
static gint timeout_tag = 0;
static GHashTable *seed_status;
static GHashTable *pushed_status;

/****************************************************************************
 * Prototypes
 ***************************************************************************/
static gchar* par_format_string(const gchar* format);
static gchar* par_format_info(void);
static void par_parsestatus(PurpleConnection *gc);
static void par_restorestatus(PurpleConnection* gc);

/****************************************************************************
 * Callbacks
 ***************************************************************************/
static void par_menu_play_cb(GtkMenuItem *item, gpointer data)
{audacious_remote_play(PAR_SESSION);
}

static void par_menu_pause_cb(GtkMenuItem *item, gpointer data)
{audacious_remote_pause(PAR_SESSION);
}

static void par_menu_stop_cb(GtkMenuItem *item, gpointer data)
{audacious_remote_stop(PAR_SESSION);
}

static void par_menu_next_cb(GtkMenuItem *item, gpointer data)
{audacious_remote_playlist_next(PAR_SESSION);
}

static void par_menu_prev_cb(GtkMenuItem *item, gpointer data)
{audacious_remote_playlist_prev(PAR_SESSION);
}

static void par_menu_repeat_cb(GtkMenuItem *item, gpointer data)
{audacious_remote_toggle_repeat(PAR_SESSION);
}

static void par_menu_shuffle_cb(GtkMenuItem *item, gpointer data)
{audacious_remote_toggle_shuffle(PAR_SESSION);
}

static void par_menu_playlist_cb(GtkMenuItem *item, gpointer data)
{gint position = GPOINTER_TO_INT(data);
 audacious_remote_set_playlist_pos(PAR_SESSION, position);
 if (!audacious_remote_is_playing(PAR_SESSION))
  audacious_remote_play(PAR_SESSION);
}

static void par_update_volume(GtkWidget *widget, gpointer null)
{gint value = (gint)gtk_range_get_value(GTK_RANGE(widget));
 audacious_remote_set_main_volume(PAR_SESSION, value);
}

static void par_button_play_cb(GtkWidget *w, GdkEventButton *e, gpointer d)
{audacious_remote_play(PAR_SESSION);
}

static void par_button_pause_cb(GtkWidget *w, GdkEventButton *e, gpointer d)
{audacious_remote_pause(PAR_SESSION);
}

static void par_button_stop_cb(GtkWidget *w, GdkEventButton *e, gpointer d)
{audacious_remote_stop(PAR_SESSION);
}

static void par_button_next_cb(GtkWidget *w, GdkEventButton *e, gpointer d)
{audacious_remote_playlist_next(PAR_SESSION);
}

static void par_button_prev_cb(GtkWidget *w, GdkEventButton *e, gpointer d)
{audacious_remote_playlist_prev(PAR_SESSION);
}

static gboolean par_watchdog_func(void) {
  GList *l;
  PurpleConnection *gc;

  if(!audacious_remote_is_playing(PAR_SESSION)) /* audacious isn't playing */
    return TRUE;
  for (l = purple_connections_get_all(); l != NULL; l = l->next) {
    gc = (PurpleConnection *)l->data;
    /* make sure we're connected */
    if (purple_connection_get_state(gc) != PURPLE_CONNECTED) {
      continue;
    }
    if (purple_prefs_get_bool("/plugins/gtk/pidgin-audacious-remote/statusmessage"))
      par_parsestatus(gc);
  }
  return TRUE;
}

static void par_restore() {
  GList *l;
  PurpleConnection *gc;

  for (l = purple_connections_get_all(); l != NULL; l = l->next) {
    gc = (PurpleConnection *)l->data;
    /* make sure we're connected */
    if (purple_connection_get_state(gc) != PURPLE_CONNECTED) {
      continue;
    }
    if (purple_prefs_get_bool("/plugins/gtk/pidgin-audacious-remote/statusmessage"))
      par_restorestatus(gc);
  }
}

/****************************************************************************
 * Helpers
 ***************************************************************************/
static void par_restorestatus(PurpleConnection* gc) {
  gchar *key = NULL;
  const gchar *seed, *proto;
  PurpleAccount *account;
  PurplePresence *presence;
  PurplePlugin *prpl;
  PurplePluginProtocolInfo *prpl_info;
  PurpleStatus *status;

  account = purple_connection_get_account(gc);
  presence = purple_account_get_presence(account);
  proto = purple_account_get_protocol_id(account);
  prpl = purple_find_prpl(proto);
  g_return_if_fail(prpl != NULL);
  prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(prpl);
  g_return_if_fail(prpl_info != NULL && prpl_info->set_status != NULL);
  status = purple_presence_get_active_status(presence);
  g_return_if_fail(status != NULL);
  /* generate key for hash table */
  key = g_strdup_printf("%s %s", account->username, account->protocol_id);
/*   par_debug("key = %s\n", key); */
  seed = (gchar*)g_hash_table_lookup(seed_status, key);
  g_return_if_fail(seed != NULL);
  par_debug("status seed = %s\n", seed);
  purple_status_set_attr_string(status, "message", seed);
  prpl_info->set_status(account, status);
  g_free(key);
}

static void par_parsestatus(PurpleConnection *gc) {
  gchar *new = NULL, *key = NULL;
  const gchar *current, *seed, *pushed, *proto;
  PurpleAccount *account;
  PurplePresence *presence;
  PurplePlugin *prpl;
  PurplePluginProtocolInfo *prpl_info;
  PurpleStatus *status;

  account = purple_connection_get_account(gc);
  presence = purple_account_get_presence(account);
  proto = purple_account_get_protocol_id(account);
  prpl = purple_find_prpl(proto);
  g_return_if_fail(prpl != NULL);
  prpl_info = PURPLE_PLUGIN_PROTOCOL_INFO(prpl);
  g_return_if_fail(prpl_info != NULL && prpl_info->set_status != NULL);
  status = purple_presence_get_active_status(presence);
  g_return_if_fail(status != NULL);
  /* generate key for hash table */
  key = g_strdup_printf("%s %s", account->username, account->protocol_id);
/*   par_debug("key = %s\n", key); */
  /* retrieve current user status */
  current = purple_status_get_attr_string(status, "message");
/*   par_debug("status current = %s\n", current); */
  /* invalidate pushded status on auto away etc. */
  if(current == NULL || strlen(current) == 0) {
    g_hash_table_replace(pushed_status, g_strdup(key), g_strdup(""));
    g_free(key);
    return;
  }
  /* pop pushed_status */
  pushed = (gchar*)g_hash_table_lookup(pushed_status, key);
/*   par_debug("status pushed = %s\n", pushed); */
  /* if current status differs from pushed_status or contains token,
   * replace hashes with current. */
  if((pushed && g_ascii_strcasecmp(current, pushed)) || strstr(current, "%") ) {
    g_hash_table_replace(seed_status, g_strdup(key), g_strdup(current));
    g_hash_table_replace(pushed_status, g_strdup(key), g_strdup(current));
    par_debug("replacing status in hashes\n");
  }
  /* construct new status message */
  seed = (gchar*)g_hash_table_lookup(seed_status, key);
  g_return_if_fail(seed != NULL);
/*   par_debug("status seed = %s\n", seed); */
  new = par_format_string(seed);
  g_return_if_fail(new != NULL);
/*   par_debug("status new = %s\n", new); */
  /* set status message only if text has been changed */
  pushed = (gchar*)g_hash_table_lookup(pushed_status, key);
/*   par_debug("status pushed = %s\n", pushed); */
  if (!pushed || g_ascii_strcasecmp(pushed, new)) {
    par_debug("setting new status for %s\n", key);
    g_hash_table_replace(pushed_status, g_strdup(key), g_strdup(new));
    purple_status_set_attr_string(status, "message", new);
    prpl_info->set_status(account, status);
  }
/*   else { */
/*     par_debug("not setting the new status\n"); */
/*   } */
  g_free(key);
  g_free(new);
}

static gchar* par_utf8unescape(const gchar* old) {
  GString* newString = g_string_new("");
  while(old[0] != '\0') {
    if (old[0] == '\\' && old[1] == '3' && old[2] == '0' && old[3] == '3') {
      g_string_append_c(newString, '\303');
      old += 5;
      guchar u = ((old[0] - '0') << 6) | ((old[1] - '0') << 3) | (old[2] - '0');
      g_string_append_c(newString, u);
      old += 3;
    }
    else {
      g_string_append_c(newString, old[0]);
      old++;
    }
  }
  gchar* retstr = newString->str;
  g_string_free(newString, FALSE);
  return retstr;
}

static gchar* par_format_string(const gchar* format) {
  GString *str;
  gchar *ret, *song = NULL;
  gchar *title = NULL, *artist = NULL, *album = NULL, *trackno = NULL;
  gchar *year = NULL, *date = NULL, *genre = NULL, *comment = NULL;
  gchar *filename = NULL, *fileext = NULL, *filepath = NULL;
  gint rate = 0, freq = 0, chan = 0;
  gint pos, length, volume, time_total, time_elapsed;
  gint min, sec;
  DBusGProxy *session = PAR_SESSION;

  pos = audacious_remote_get_playlist_pos(session);
  time_total = audacious_remote_get_playlist_time(session, pos);
  time_elapsed = audacious_remote_get_output_time(session);
  audacious_remote_get_info(session, &rate, &freq, &chan);
  length = audacious_remote_get_playlist_length(session);
  volume = audacious_remote_get_main_volume(session);
  song = audacious_remote_get_playlist_title(session, pos);
  str = g_string_new("");
  par_debug("format=%s, len=%d\n", format, (unsigned int)strlen(format));
  while (format[0]) {
    if (format[0] != '%') {
      str = g_string_append_c(str, format[0]);
      format++;
      continue;
    }
    format++;
/*     if (!format[0]) */
/*       break; */
    par_debug("format[0]=%c\n", format[0]);
    switch (format[0]) {
    case 'n':
      title = audacious_get_tuple_field_data(session, "title", pos);
      if (title) {
	gchar* utf8title = par_utf8unescape(title);
        g_string_append(str, utf8title);
	g_free(utf8title);
        g_free(title);
      }
      break;
    case 'a':
      artist = audacious_get_tuple_field_data(session, "artist", pos);
      if (artist) {
	gchar* utf8artist = par_utf8unescape(artist);
        g_string_append(str, utf8artist);
	g_free(utf8artist);
        g_free(artist);
      }
      break;
    case 'c':
      album = audacious_get_tuple_field_data(session, "album", pos);
      if (album) {
	gchar* utf8album = par_utf8unescape(album);
        g_string_append(str, utf8album);
	g_free(utf8album);
        g_free(album);
      }
      break;
    case 'd':
      trackno = audacious_get_tuple_field_data(session, "track_number", pos);
      if (trackno) {
        g_string_append(str, trackno);
        g_free(trackno);
      }
      break;
    case 'y':
      year = audacious_get_tuple_field_data(session, "year", pos);
      if (year) {
        g_string_append(str, year);
        g_free(year);
      }
      break;
    case 'Y':
      date = audacious_get_tuple_field_data(session, "date", pos);
      if (date) {
        g_string_append(str, date);
        g_free(date);
      }
      break;
    case 'g':
      comment = audacious_get_tuple_field_data(session, "comment", pos);
      if (comment) {
	gchar* utf8comment = par_utf8unescape(comment);
        g_string_append(str, utf8comment);
	g_free(utf8comment);
        g_free(comment);
      }
      break;
    case 'N':
      filename = audacious_get_tuple_field_data(session, "file_name", pos);
      if (filename) {
	gchar* utf8filename = par_utf8unescape(filename);
        g_string_append(str, utf8filename);
	g_free(utf8filename);
        g_free(filename);
      }
      break;
    case 'E':
      fileext = audacious_get_tuple_field_data(session, "file_ext", pos);
      if (fileext) {
	gchar* utf8fileext = par_utf8unescape(fileext);
        g_string_append(str, utf8fileext);
	g_free(utf8fileext);
        g_free(fileext);
      }
      break;
    case 'A':
      filepath = audacious_get_tuple_field_data(session, "file_path", pos);
      if (filepath) {
	gchar* utf8filepath = par_utf8unescape(filepath);
        g_string_append(str, utf8filepath);
	g_free(utf8filepath);
        g_free(filepath);
      }
      break;
    case 'C':
      g_string_append_printf(str, "%d", chan);
      break;
    case 'F':
      g_string_append_printf(str, "%g", (gfloat)freq / 1000.0f);
      break;
    case 'f':
      g_string_append_printf(str, "%d", freq);
      break;
    case 'L':
      g_string_append_printf(str, "%d", length);
      break;
    case 'P':
      g_string_append_printf(str, "%d", pos + 1);
      break;
    case 'B':
      g_string_append_printf(str, "%g", (gfloat)rate / 1000.0f);
      break;
    case 'b':
      g_string_append_printf(str, "%d", rate);
      break;
    case 'T':
      str = g_string_append(str, song);
      break;
    case 'V':
      g_string_append_printf(str, "%d", volume);
      break;
    case 't':
      min = (gint)(time_total / 60000);
      sec = (gint)(time_total / 1000 % 60);
      g_string_append_printf(str, "%d:%02d", min, sec);
      break;
    case 'e':
      min = (gint)(time_elapsed / 60000);
      sec = (gint)(time_elapsed / 1000 % 60);
      g_string_append_printf(str, "%d:%02d", min, sec);
      break;
    case 'r':
      min = (gint)((time_total - time_elapsed) / 60000);
      sec = (gint)((time_total - time_elapsed) / 1000 % 60);
      g_string_append_printf(str, "%d:%02d", min, sec);
      break;
    case '%':
    default:
      str = g_string_append_c(str, '%');
      break;
    }
    format++;
  }

/*   ret = g_markup_escape_text(str->str, strlen(str->str)); */
  ret = str->str;
  g_string_free(str, FALSE);
  if (song)
    g_free(song);
  return ret;
}

static gchar* par_format_info() {
  const gchar* format = purple_prefs_get_string("/plugins/gtk/pidgin-audacious-remote/format");
  return par_format_string(format);
}

static void par_display_title(PidginWindow *win)
{PurpleConversationType type;
 gchar *text = NULL;
 PurpleConversation *conv;

 g_return_if_fail(win);
 conv = pidgin_conv_window_get_active_conversation(win);
 type = purple_conversation_get_type(conv);
 text = par_format_info();
 if (!text)
  return;
 switch (type)
 {case PURPLE_CONV_TYPE_IM:
   purple_conv_im_send(PURPLE_CONV_IM(conv), text);
   break;
  case PURPLE_CONV_TYPE_CHAT:
   purple_conv_chat_send(PURPLE_CONV_CHAT(conv), text);
   break;
  case PURPLE_CONV_TYPE_UNKNOWN:
  case PURPLE_CONV_TYPE_MISC:
  default:
   break;
 }
 if (text)
  g_free(text);
}

static void par_menu_display_title_cb(GtkMenuItem *item, gpointer data)
{PidginWindow *win;

 win = (PidginWindow*)data;
 par_display_title(win);
}

static GtkWidget* par_make_item(GtkWidget *menu, const gchar *text, GtkSignalFunc sf, gpointer data)
{GtkWidget *item;

 item = gtk_menu_item_new_with_label(text);
 if (menu != NULL)
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 gtk_widget_show(item);
 if (sf != NULL)
  g_signal_connect(G_OBJECT(item), "activate", sf, data);
 return item;
}

static GtkWidget* par_make_button(const char* stock, GCallback cb, gpointer data, PidginWindow *win)
{GtkWidget *ebox, *image;

 ebox = gtk_event_box_new();
 gtk_widget_show(ebox);
 image = gtk_image_new_from_stock(stock, GTK_ICON_SIZE_MENU);
 gtk_container_add(GTK_CONTAINER(ebox), image);
 gtk_widget_show(image);
 g_signal_connect(G_OBJECT(ebox), "button_press_event", G_CALLBACK(cb), data);
 g_object_set_data(G_OBJECT(ebox), "win", win);
 return ebox;
}

static void par_make_playlist(GtkWidget* menu_item)
{GtkWidget *menu, *item;
 gint i, count, current;
 DBusGProxy *session = PAR_SESSION;
 gchar *title, *song;

 menu = gtk_menu_new();
 count = audacious_remote_get_playlist_length(session);
 current = audacious_remote_get_playlist_pos(session);
 for(i = 0; i < count; i++)
 {song = audacious_remote_get_playlist_title(session, i);
  title = g_strdup_printf("%d. %s", i + 1, song);
  g_free(song);
  if (i == current)
   pidgin_new_check_item(menu, title, G_CALLBACK(par_menu_playlist_cb), GINT_TO_POINTER(i), TRUE);
  else
   item = par_make_item(menu, title, G_CALLBACK(par_menu_playlist_cb), GINT_TO_POINTER(i));
  g_free(title);
 }
 gtk_widget_show_all(menu);
 gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), menu);
 if (count == 0)
  gtk_widget_set_sensitive(menu_item, FALSE);
}

static GtkWidget* par_make_menu(PidginWindow *win)
{GtkWidget *menu, *item;
 DBusGProxy *session = PAR_SESSION;

 menu = gtk_menu_new();
 if (!audacious_remote_is_running(session))
 {item = pidgin_new_item_from_stock(menu, "Please start Audacious", PAR_STOCK_AUDACIOUS, NULL, NULL, 0, 0, NULL);
  gtk_widget_set_sensitive(item, FALSE);
  return menu;
 }
 /* play */
 item = pidgin_new_item_from_stock(menu, "Play", PAR_STOCK_PLAY, G_CALLBACK(par_menu_play_cb), NULL, 0, 0, NULL);
 if (audacious_remote_is_playing(session) && !audacious_remote_is_paused(session))
  gtk_widget_set_sensitive(item, FALSE);
 /* pause */
 item = pidgin_new_item_from_stock(menu, "Pause", PAR_STOCK_PAUSE, G_CALLBACK(par_menu_pause_cb), NULL, 0, 0, NULL);
 if (!audacious_remote_is_playing(session) && !audacious_remote_is_paused(session))
  gtk_widget_set_sensitive(item, FALSE);
 if (audacious_remote_is_paused(session))
  gtk_widget_set_sensitive(item, FALSE);
 /* stop */
 item = pidgin_new_item_from_stock(menu, "Stop", PAR_STOCK_STOP, G_CALLBACK(par_menu_stop_cb), NULL, 0, 0, NULL);
 if (!audacious_remote_is_playing(session) && !audacious_remote_is_paused(session))
  gtk_widget_set_sensitive(item, FALSE);
 /* next */
 pidgin_new_item_from_stock(menu, "Next", PAR_STOCK_NEXT, G_CALLBACK(par_menu_next_cb), NULL, 0, 0, NULL);
 /* previous */
 pidgin_new_item_from_stock(menu, "Previous", PAR_STOCK_PREVIOUS, G_CALLBACK(par_menu_prev_cb), NULL, 0, 0, NULL);
 /* separator */
 pidgin_separator(menu);
 /* repeat */
 pidgin_new_check_item(menu, "Repeat", G_CALLBACK(par_menu_repeat_cb), NULL, audacious_remote_is_repeat(session));
 /* shuffle */
 pidgin_new_check_item(menu, "Shuffle", G_CALLBACK(par_menu_shuffle_cb), NULL, audacious_remote_is_shuffle(session));
 if (purple_prefs_get_bool("/plugins/gtk/pidgin-audacious-remote/show_playlist"))
 {/* separator */
  pidgin_separator(menu);
  /* playlist */
  item = par_make_item(menu, "Playlist", NULL, NULL);
  par_make_playlist(item);
 }
 if (win)
 {/* Only if we are in a conversation window */
  /* separator */
  pidgin_separator(menu);
  /* title */
  item = par_make_item(menu, "Display title", G_CALLBACK(par_menu_display_title_cb), (gpointer)win);
 }
 return menu;
}

static void par_button_clicked_cb(GtkButton *button, gpointer data)
{PidginWindow *win;
 GtkWidget *menu;

 win = (PidginWindow*)g_object_get_data(G_OBJECT(button), "win");
 menu = par_make_menu(win);
 if (win)
  gtk_widget_grab_focus(pidgin_conv_window_get_active_gtkconv(win)->entry);
 gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time());
}

static void par_add_button(PidginWindow *win)
{PidginConversation *gtkconv = pidgin_conv_window_get_active_gtkconv(win);
 PurpleConversation *conv = gtkconv->active_conv;
 PurpleConversationType type = purple_conversation_get_type(conv);
 GtkWidget *tray;
 GtkWidget *button = NULL;
 GList *l;

 if (type != PURPLE_CONV_TYPE_IM && type != PURPLE_CONV_TYPE_CHAT)
  return;
 if (!purple_prefs_get_bool("/plugins/gtk/pidgin-audacious-remote/conv"))
  return;
 for (l = buttons; l; l = l->next)
 {button = (GtkWidget*)l->data;
  if (g_object_get_data(G_OBJECT(button), "win") == win)
   return;
 }
 tray = win->menu.tray;
 if (!purple_prefs_get_bool("/plugins/gtk/pidgin-audacious-remote/extended"))
 {/* Show the minimal control */
  button = par_make_button(PAR_STOCK_AUDACIOUS, G_CALLBACK(par_button_clicked_cb), win, win);
  pidgin_menu_tray_append(PIDGIN_MENU_TRAY(win->menu.tray), button, "Pidgin-Audacious-Remote Options");
  buttons = g_list_append(buttons, (gpointer)button);
 }
 else
 {/* Show extended control */
  /* we're using gtk_menu_shell_append, so these need to be added in
   * reverse order
   */
  button = par_make_button(PAR_STOCK_NEXT, G_CALLBACK(par_button_next_cb), NULL, win);
  pidgin_menu_tray_append(PIDGIN_MENU_TRAY(win->menu.tray), button, "Next");
  buttons = g_list_append(buttons, (gpointer)button);
  button = par_make_button(PAR_STOCK_STOP, G_CALLBACK(par_button_stop_cb), NULL, win);
  pidgin_menu_tray_append(PIDGIN_MENU_TRAY(win->menu.tray), button, "Stop");
  buttons = g_list_append(buttons, (gpointer)button);
  button = par_make_button(PAR_STOCK_PAUSE, G_CALLBACK(par_button_pause_cb), NULL, win);
  pidgin_menu_tray_append(PIDGIN_MENU_TRAY(win->menu.tray), button, "Pause");
  buttons = g_list_append(buttons, (gpointer)button);
  button = par_make_button(PAR_STOCK_PLAY, G_CALLBACK(par_button_play_cb), NULL, win);
  pidgin_menu_tray_append(PIDGIN_MENU_TRAY(win->menu.tray), button, "Play");
  buttons = g_list_append(buttons, (gpointer)button);
  button = par_make_button(PAR_STOCK_PREVIOUS, G_CALLBACK(par_button_prev_cb), NULL, win);
  pidgin_menu_tray_append(PIDGIN_MENU_TRAY(win->menu.tray), button, "Previous");
  buttons = g_list_append(buttons, (gpointer)button);
  if (purple_prefs_get_bool("/plugins/gtk/pidgin-audacious-remote/volume"))
  {/* The volume controller */
   GtkWidget *slider;
   slider = gtk_hscale_new_with_range(0, 100, 1);
   gtk_widget_set_usize(GTK_WIDGET(slider), 100, -1);
   gtk_scale_set_draw_value(GTK_SCALE(slider), FALSE);
   gtk_range_set_value(GTK_RANGE(slider), audacious_remote_get_main_volume(PAR_SESSION));
   pidgin_menu_tray_append(PIDGIN_MENU_TRAY(win->menu.tray), slider, "Audacious Volume Control");
   g_object_set_data(G_OBJECT(slider), "win", win);
   buttons = g_list_prepend(buttons, slider);
   g_signal_connect(G_OBJECT(slider), "value-changed", G_CALLBACK(par_update_volume), NULL);
   gtk_widget_show(slider);
  }
 }
}

static void par_show_buttons()
{GList *wins;

 for (wins = pidgin_conv_windows_get_list(); wins; wins = wins->next)
  par_add_button((PidginWindow*)(wins->data));
}

static void par_hide_buttons()
{PidginWindow *win;
 GtkWidget *button;
 GList *l, *l_next;

 for(l = buttons; l; l = l_next)
 {l_next = l->next;
  button = GTK_WIDGET(l->data);
  win = (PidginWindow*)g_object_get_data(G_OBJECT(button), "win");
  if (!win)
   buttons = g_list_remove(buttons, button);
  else
  {gtk_widget_destroy(button);
   buttons = g_list_remove(buttons, button);
  }
 }
}

static gboolean is_active(GList *list, gint id)
{for (; list; list = list->next)
 {int itmp = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(list->data), "par-id"));
  if (itmp == id)
   return (GTK_WIDGET_IS_SENSITIVE(GTK_WIDGET(list->data)) && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(list->data)));
 }
 return FALSE;
}

static void set_active(GList *list, gint id, gboolean active)
{for (; list; list = list->next)
 {int itmp = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(list->data), "par-id"));
  if (itmp == id)
  {gtk_widget_set_sensitive(GTK_WIDGET(list->data), active);
   break;
  }
 }
}

static void refresh_buttons()
{if (is_active(checkboxes, PAR_PREF_CONV_WINDOW))
 {set_active(checkboxes, PAR_PREF_EXTENDED_CTRL, TRUE);
  if (is_active(checkboxes, PAR_PREF_EXTENDED_CTRL))
   set_active(checkboxes, PAR_PREF_VOLUME_CTRL, TRUE);
  else
   set_active(checkboxes, PAR_PREF_VOLUME_CTRL, FALSE);
 }
 else
 {set_active(checkboxes, PAR_PREF_EXTENDED_CTRL, FALSE);
  set_active(checkboxes, PAR_PREF_VOLUME_CTRL, FALSE);
 }
}

static void par_button_show_cb(const char *name, PurplePrefType type, gconstpointer val, gpointer data)
{refresh_buttons();
 par_hide_buttons();
 par_show_buttons();
}

static void par_popup_cb(GtkWidget *w, GtkMenu *menu, gpointer data)
{GtkWidget *item, *submenu;

 pidgin_separator(GTK_WIDGET(menu));
 item = pidgin_new_item_from_stock(GTK_WIDGET(menu), "Pidgin-Audacious-Remote", PAR_STOCK_AUDACIOUS, NULL, NULL, 0, 0, NULL);
 submenu = par_make_menu((PidginWindow*)data);
 gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
 gtk_widget_show(item);
}

void par_disconnect_popup_cb(PurpleConversation *conv)
{PidginWindow *win;
 PidginConversation *gtkconv;
 GtkWidget *entry;
 gulong handle;

 if ((gtkconv = PIDGIN_CONVERSATION(conv)) == NULL)
  return;
 win = pidgin_conv_get_window(gtkconv);
 entry = gtkconv->entry;
 handle = (gulong)g_object_get_data(G_OBJECT(entry), "par-popup-handle");
 if (handle == 0)
  return;
 g_signal_handler_disconnect(G_OBJECT(entry), handle);
 g_object_set_data(G_OBJECT(entry), "par-popup-handle", 0);
}

static void par_hook_popup_for_gtkconv(PidginConversation *gtkconv)
{gulong handle;
 GtkWidget *entry;
 PidginWindow *win;

 win = pidgin_conv_get_window(gtkconv);
 entry = gtkconv->entry;
 if ((gulong)g_object_get_data(G_OBJECT(entry), "par-popup-handle"))
  return;
 handle = g_signal_connect(G_OBJECT(entry), "populate-popup", G_CALLBACK(par_popup_cb), win);
 g_object_set_data(G_OBJECT(entry), "par-popup-handle", (gpointer)handle);
}

static void par_hook_popups()
{GList *convs;
 PurpleConversation *conv;

 for (convs = purple_get_conversations(); convs; convs = convs->next)
 {PidginConversation *gtkconv;
  conv = (PurpleConversation*)convs->data;
  gtkconv = PIDGIN_CONVERSATION(conv);
  par_hook_popup_for_gtkconv(gtkconv);
 }
}

static gboolean attach_to_window_tray(PurpleConversation *conv)
{PidginConversation *gtkconv;
 PidginWindow *win;

 if ((gtkconv = PIDGIN_CONVERSATION(conv)) == NULL)
  return TRUE;
 win = pidgin_conv_get_window(gtkconv);
 if (!win)
  return TRUE;
 if (!win->window)
  return TRUE;
 if (!GTK_WIDGET_VISIBLE(win->window))
  return TRUE;
 par_add_button(win);
 par_hook_popup_for_gtkconv(gtkconv);
 return FALSE;
}

static void par_conv_created_cb(PurpleConversation *conv, gpointer data)
{/* This terrible workaround is necessary because when you have
  * message-queueing enabled, the conversation does NOT get attached
  * to the viewable window immediately when it gets created. Rather,
  * it is shown only after the signal has been emitted. There might
  * be a better way of doing this. */
 g_timeout_add(50, (GSourceFunc)attach_to_window_tray, conv);
}

static void par_conv_destroyed_cb(PurpleConversation *conv, gpointer data)
{PidginWindow *win;
 GtkWidget *button;
 GList *l, *l_next;

 win = pidgin_conv_get_window(PIDGIN_CONVERSATION(conv));
 if (pidgin_conv_window_get_gtkconv_count(win) != 1)
  return;
 for (l = buttons; l != NULL; l = l_next)
 {l_next = l->next;
  button = GTK_WIDGET(l->data);
  if (win == g_object_get_data(G_OBJECT(button), "win"))
  {gtk_widget_destroy(button);
   buttons = g_list_remove(buttons, l->data);
  }
 }
}

static PurpleCmdRet par_cmd_cb(PurpleConversation *c, const gchar *cmd, gchar **args, gchar **error, void *data)
{gchar *lower;
 DBusGProxy *session = PAR_SESSION;
 PidginWindow *win;

 win = pidgin_conv_get_window(PIDGIN_CONVERSATION(c));
 if (!audacious_remote_is_running(session))
 {*error = g_strdup("Audacious is not running");
  return PURPLE_CMD_RET_FAILED;
 }
 if (!*args && !args [0])
 {*error = g_strdup("eek!");
  return PURPLE_CMD_RET_FAILED;
 }
 lower = g_ascii_strdown(args[0], strlen(args[0]));
 if (strcmp(lower, "play") == 0)
  audacious_remote_play(session);
 else if (!strcmp(lower, "pause"))
  audacious_remote_pause(session);
 else if (!strcmp(lower, "stop"))
  audacious_remote_stop(session);
 else if (!strcmp(lower, "next"))
  audacious_remote_playlist_next(session);
 else if (!strcmp(lower, "prev"))
  audacious_remote_playlist_prev(session);
 else if (!strcmp(lower, "info"))
  par_display_title(win);
 else if (!strcmp(lower, "repeat"))
  audacious_remote_toggle_repeat(session);
 else if (!strcmp(lower, "shuffle"))
  audacious_remote_toggle_shuffle(session);
 else if (!strcmp(lower, "show"))
  audacious_remote_main_win_toggle(session, TRUE);
 else if (!strcmp(lower, "hide"))
  audacious_remote_main_win_toggle(session, FALSE);
 else
 {*error = g_strdup("unknown argument");
  return PURPLE_CMD_RET_FAILED;
 }
 g_free(lower);
 return PURPLE_CMD_RET_OK;
}

static void par_removekey(gpointer data) {
  g_free(data);
}

static void par_removeval(gpointer data) {
  g_free(data);
}

/****************************************************************************
 * Prefs stuff
 ***************************************************************************/
static GtkWidget* par_make_label(const gchar *text, GtkSizeGroup *sg)
{GtkWidget *label;

 label = gtk_label_new_with_mnemonic(text);
 gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 gtk_widget_show(label);
 if (sg)
  gtk_size_group_add_widget(sg, label);
 return label;
}

static GtkWidget* par_get_config_frame(PurplePlugin *plugin)
{GtkWidget *vbox, *hbox, *frame, *label, *checkbox;
 GtkSizeGroup *sg;

 vbox = gtk_vbox_new(FALSE, 6);
 gtk_container_set_border_width(GTK_CONTAINER(vbox), 12);
 frame = pidgin_make_frame(vbox, "Info");
 pidgin_prefs_labeled_entry(frame, "Info Format:", "/plugins/gtk/pidgin-audacious-remote/format", NULL);
 sg = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);

 hbox = gtk_hbox_new(FALSE, 6);
 gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
 gtk_widget_show(hbox);
 label = par_make_label("%n: Song title", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 label = par_make_label("%a: Artist", NULL);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

 hbox = gtk_hbox_new(FALSE, 6);
 gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
 gtk_widget_show(hbox);
 label = par_make_label("%c: Album", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 label = par_make_label("%d: Track number", NULL);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

 hbox = gtk_hbox_new(FALSE, 6);
 gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
 gtk_widget_show(hbox);
 label = par_make_label("%y: Song year", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 label = par_make_label("%Y: Date", NULL);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

 hbox = gtk_hbox_new(FALSE, 6);
 gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
 gtk_widget_show(hbox);
 label = par_make_label("%g: Comment", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 label = par_make_label("%N: File name", NULL);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

 hbox = gtk_hbox_new(FALSE, 6);
 gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
 gtk_widget_show(hbox);
 label = par_make_label("%E: File extension", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 label = par_make_label("%A: File path", NULL);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

 hbox = gtk_hbox_new(FALSE, 6);
 gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
 gtk_widget_show(hbox);
 label = par_make_label("%T: Artist and title", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 label = par_make_label("%C: Number of channels", NULL);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

 hbox = gtk_hbox_new(FALSE, 6);
 gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
 gtk_widget_show(hbox);
 label = par_make_label("%P: Current song playlist number", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 label = par_make_label("%L: Total songs in the playlist", NULL);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

 hbox = gtk_hbox_new(FALSE, 6);
 gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
 gtk_widget_show(hbox);
 label = par_make_label("%t: Total time", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 label = par_make_label("%e: Elapsed time", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

 hbox = gtk_hbox_new(FALSE, 6);
 gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
 gtk_widget_show(hbox);
 label = par_make_label("%r: Remaining time", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 label = par_make_label("%V: Current volume", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

 hbox = gtk_hbox_new(FALSE, 6);
 gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
 gtk_widget_show(hbox);
 label = par_make_label("%f: Frequency in Hz", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 label = par_make_label("%F: Frequency in kHz", NULL);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

 hbox = gtk_hbox_new(FALSE, 6);
 gtk_box_pack_start(GTK_BOX(frame), hbox, FALSE, FALSE, 0);
 gtk_widget_show(hbox);
 label = par_make_label("%b: Bitrate in bps", sg);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 label = par_make_label("%B: Bitrate in kBps", NULL);
 gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

 frame = pidgin_make_frame(vbox, "Appearance");
 checkbox = pidgin_prefs_checkbox("Show playlist in the control menu", "/plugins/gtk/pidgin-audacious-remote/show_playlist", frame);
 g_object_set_data(G_OBJECT(checkbox), "par-id", GINT_TO_POINTER(PAR_PREF_PLAYLIST));
 checkboxes = g_list_prepend(checkboxes, checkbox);
 checkbox = pidgin_prefs_checkbox("Show controls in buddy list", "/plugins/gtk/pidgin-audacious-remote/blist", frame);
 g_object_set_data(G_OBJECT(checkbox), "par-id", GINT_TO_POINTER(PAR_PREF_BLIST));
 checkboxes = g_list_prepend(checkboxes, checkbox);
 checkbox = pidgin_prefs_checkbox("Show controls in conversation windows", "/plugins/gtk/pidgin-audacious-remote/conv", frame);
 g_object_set_data(G_OBJECT(checkbox), "par-id", GINT_TO_POINTER(PAR_PREF_CONV_WINDOW));
 checkboxes = g_list_prepend(checkboxes, checkbox);
 checkbox = pidgin_prefs_checkbox("Show extended controls (Conversation windows only)", "/plugins/gtk/pidgin-audacious-remote/extended", frame);
 g_object_set_data(G_OBJECT(checkbox), "par-id", GINT_TO_POINTER(PAR_PREF_EXTENDED_CTRL));
 checkboxes = g_list_prepend(checkboxes, checkbox);
 checkbox = pidgin_prefs_checkbox("Show volume control (Conversation windows only)", "/plugins/gtk/pidgin-audacious-remote/volume", frame);
 g_object_set_data(G_OBJECT(checkbox), "par-id", GINT_TO_POINTER(PAR_PREF_VOLUME_CTRL));
 checkboxes = g_list_prepend(checkboxes, checkbox);
 frame = pidgin_make_frame(vbox, "Status message");
 pidgin_prefs_checkbox("Parse current status message", "/plugins/gtk/pidgin-audacious-remote/statusmessage", frame);
 refresh_buttons();
 gtk_widget_show_all(vbox);
 return vbox;
}

/****************************************************************************
 * Stock stuff
 ***************************************************************************/
/**
 * FIXME: The path should not be static, but built at compile time or runtime.
 */
static gchar* par_file_name(const gchar *file_name)
{return g_build_filename("/usr/share", "pixmaps", "pidgin", "pidgin-audacious-remote", file_name, NULL);
}

static const GtkStockItem stock_items[] =
{{PAR_STOCK_AUDACIOUS, "", 0, 0, NULL},
 {PAR_STOCK_PREVIOUS, "", 0, 0, NULL},
 {PAR_STOCK_PLAY, "", 0, 0, NULL},
 {PAR_STOCK_PAUSE, "", 0, 0, NULL},
 {PAR_STOCK_STOP, "", 0, 0, NULL},
 {PAR_STOCK_NEXT, "", 0, 0, NULL}
};

static void par_add_to_stock(const gchar *file_name, const gchar *stock_name)
{GtkIconSet *icon_set;
 GdkPixbuf *pixbuf;
 gchar *file = par_file_name(file_name);

 pixbuf = gdk_pixbuf_new_from_file(file, NULL);
 g_free(file);
 icon_set = gtk_icon_set_new_from_pixbuf(pixbuf);
 g_object_unref(G_OBJECT(pixbuf));
 gtk_icon_factory_add(icon_factory, stock_name, icon_set);
 gtk_icon_set_unref(icon_set);
}

static void par_init_stock()
{icon_factory = gtk_icon_factory_new();
 gtk_icon_factory_add_default(icon_factory);
 par_add_to_stock("next.png", PAR_STOCK_NEXT);
 par_add_to_stock("pause.png", PAR_STOCK_PAUSE);
 par_add_to_stock("play.png", PAR_STOCK_PLAY);
 par_add_to_stock("previous.png", PAR_STOCK_PREVIOUS);
 par_add_to_stock("stop.png", PAR_STOCK_STOP);
 par_add_to_stock("audacious.png", PAR_STOCK_AUDACIOUS);
 gtk_stock_add_static(stock_items, G_N_ELEMENTS(stock_items));
}

static void par_hook_blist(const char *name, PurplePrefType type, gconstpointer val, gpointer data)
{gboolean value = GPOINTER_TO_INT(val);

 if (value && !blist_button)
 {PidginBuddyList *gtkblist = pidgin_blist_get_default_gtk_blist();
  blist_button = par_make_button(PAR_STOCK_AUDACIOUS, G_CALLBACK(par_button_clicked_cb), NULL, NULL);
  pidgin_menu_tray_append(PIDGIN_MENU_TRAY(gtkblist->menutray), blist_button, "Pidgin-Audacious-Remote Options");
 }
 else
 {if (blist_button)
  {gtk_widget_destroy(blist_button);
   blist_button = NULL;
  }
 }
}

static void par_gtkblist_created_cb()
{purple_prefs_trigger_callback("/plugins/gtk/pidgin-audacious-remote/blist");
}

/****************************************************************************
 * Plugin stuff
 ***************************************************************************/

static gboolean par_load(PurplePlugin *plugin)
{void *conv_handle = purple_conversations_get_handle();
 const gchar *help = "<pre>par &lt;[play][pause][stop][next][prev][repeat][shuffle][show][hide][info]&gt;\n"
  "Play     Starts playback\n"
  "Pause    Pauses playback\n"
  "Stop     Stops playback\n"
  "Next     Goes to the next song in the playlist\n"
  "Prev     Goes to the previous song in the playlist\n"
  "Repeat   Toggles repeat\n"
  "Shuffle  Toggles shuffling\n"
  "Show     Show the Audacious window\n"
  "Hide     Hide the Audacious window\n"
  "Info     Displays currently playing song in the conversation</pre>";

 /* init our stock */
 par_init_stock();
 /* connect to the signals we need.. */
 purple_signal_connect(conv_handle, "conversation-created", plugin, PURPLE_CALLBACK(par_conv_created_cb), NULL);
 purple_signal_connect(conv_handle, "deleting-conversation", plugin, PURPLE_CALLBACK(par_conv_destroyed_cb), NULL);
 purple_prefs_connect_callback(plugin, "/plugins/gtk/pidgin-audacious-remote/conv", par_button_show_cb, NULL);
 purple_prefs_connect_callback(plugin, "/plugins/gtk/pidgin-audacious-remote/blist", par_hook_blist, NULL);
 purple_prefs_connect_callback(plugin, "/plugins/gtk/pidgin-audacious-remote/extended", par_button_show_cb, NULL);
 purple_prefs_connect_callback(plugin, "/plugins/gtk/pidgin-audacious-remote/volume", par_button_show_cb, NULL);
 par_show_buttons();
 /* register our command */
 par_cmd = purple_cmd_register("par", "w", PURPLE_CMD_P_PLUGIN, (PurpleCmdFlag)(PURPLE_CMD_FLAG_IM | PURPLE_CMD_FLAG_CHAT), NULL, par_cmd_cb, help, NULL);
 /* hijack the popup menus in the imhtmls */
 par_hook_popups();
 if (purple_prefs_get_bool("/plugins/gtk/pidgin-audacious-remote/blist") && !pidgin_blist_get_default_gtk_blist())
  purple_signal_connect(pidgin_blist_get_handle(), "gtkblist-created", plugin, PURPLE_CALLBACK(par_gtkblist_created_cb), NULL);
 else
  purple_prefs_trigger_callback("/plugins/gtk/pidgin-audacious-remote/blist");

 /* Connecting to DBus for Audacious */
 GError *error = NULL;
 DBusGConnection *connection = dbus_g_bus_get(DBUS_BUS_SESSION, &error);
 if (connection == NULL)
   return FALSE;
 PAR_SESSION = dbus_g_proxy_new_for_name(connection, AUDACIOUS_DBUS_SERVICE, AUDACIOUS_DBUS_PATH, AUDACIOUS_DBUS_INTERFACE);

 /* Creating hashes for status message */
 seed_status = g_hash_table_new_full(g_str_hash, g_str_equal, par_removekey, par_removeval);
 pushed_status = g_hash_table_new_full(g_str_hash, g_str_equal, par_removekey, par_removeval);
 timeout_tag = g_timeout_add(PAR_DBUS_TIMEOUT, (gpointer) par_watchdog_func, NULL);
/*  purple_signal_connect(purple_connections_get_handle(), "signing-off", plugin, PURPLE_CALLBACK(restore_alias), NULL); */

 return TRUE;
}

static gboolean par_unload(PurplePlugin *plugin) {
  /* Delete status message timer, restore old status message and destroy hashes */
  par_restore();
  if (timeout_tag) g_source_remove(timeout_tag);
  timeout_tag = 0;
  g_hash_table_destroy(seed_status);
  g_hash_table_destroy(pushed_status);

  /* remove our buttons */
  par_hide_buttons();
  g_list_free(buttons);
  buttons = NULL;
  if (blist_button)
  {gtk_widget_destroy(blist_button);
   blist_button = NULL;
  }
  /* remove our popup menu item */
  purple_conversation_foreach(par_disconnect_popup_cb);
  /* remove our icons */
  gtk_icon_factory_remove_default(icon_factory);
  /* remove our command */
  purple_cmd_unregister(par_cmd);
  return TRUE;
}

static void init_plugin(PurplePlugin *plugin)
{purple_prefs_add_none("/plugins/gtk");
 purple_prefs_add_none("/plugins/gtk/pidgin-audacious-remote");
 purple_prefs_add_string("/plugins/gtk/pidgin-audacious-remote/format", "/me is listening to %T");
 purple_prefs_add_bool("/plugins/gtk/pidgin-audacious-remote/show_playlist", TRUE);
 purple_prefs_add_bool("/plugins/gtk/pidgin-audacious-remote/conv", TRUE);
 purple_prefs_add_bool("/plugins/gtk/pidgin-audacious-remote/blist", TRUE);
 purple_prefs_add_bool("/plugins/gtk/pidgin-audacious-remote/extended", TRUE);
 purple_prefs_add_bool("/plugins/gtk/pidgin-audacious-remote/volume", TRUE);
 purple_prefs_add_bool("/plugins/gtk/pidgin-audacious-remote/statusmessage", TRUE);
}

static PidginPluginUiInfo ui_info = {
 par_get_config_frame,
 0,
 NULL,
 NULL,
 NULL,
 NULL
};

static PurplePluginInfo par_info =
{PURPLE_PLUGIN_MAGIC,
 PURPLE_MAJOR_VERSION,
 PURPLE_MINOR_VERSION,
 PURPLE_PLUGIN_STANDARD, /* type */
 PIDGIN_PLUGIN_TYPE, /* ui requirement */
 0, /* flags */
 NULL, /* dependencies */
 PURPLE_PRIORITY_DEFAULT, /* priority */
 "gtk-pidgin_audacious_remote", /* id */
 "Pidgin-Audacious-Remote", /* name */
 PAR_VERSION, /* version */
 "Control Audacious from Pidgin conversations", /* summary */
 "A small plugin that adds a button to the "
 "pidgin conversation windows, so that you "
 "can control Audacious from within Pidgin. "
 "Plus, it allows you to automatically update "
 "your status message with songs' informations.", /* description */
 "Francesco Russo <francescor82@salug.it>", /* author */
 "http://pidginaudacious.sourceforge.net/", /* homepage */
 par_load, /* load */
 par_unload, /* unload */
 NULL, /* destroy */
 &ui_info, /* ui info */
 NULL, /* extra info */
 NULL, /* actions info */
 NULL, /* actions info */
 NULL, /* reserved 1 */
 NULL, /* reserved 2 */
 NULL, /* reserved 3 */
 NULL /* reserved 4 */
};

PURPLE_INIT_PLUGIN(Pidgin-Audacious-Remote, init_plugin, par_info)
